//******************************************************************************
// TCP Plugin Implemenatation
//
// - this unit imiplements the TCP connection
// - the protocol specific chat rules are implemented in
//   the unProtoImpl unit
//******************************************************************************

unit unImpl;


interface
uses classes,windows,extCtrls;

const
   eol = chr(13)+chr(10);

type
    TPluginImpl = class(TObject)
    private
    url   : string;
    timer : TTimer;
    procedure Timer1Timer(Sender: TObject);
    function findFile(id:string) : string;
    public
    owner : pointer;
    function getNiceName : string;
    procedure Init;
    procedure connect;
    procedure disconnect;

    procedure onEnvelopeReceived(envelopeXML : string);
    procedure onEnvelopeSent(envelopeID : string);
    procedure onEnvelopeDeleted(envelopeID : string);
    procedure onCheckDirTimer;

    constructor Create;
    destructor Destroy; override;
    end;


implementation
uses unPlugin,sysutils,unEnvelope,unHTTPCLient,ozUrl,unStrings,unGsmSMS,
     unFileSystem,unDate,unMain;

constructor TPluginImpl.Create;
Begin
    owner := nil;
    timer := TTimer.Create(nil);
    timer.Enabled := false;
    timer.OnTimer := Timer1Timer;
End;

Procedure TPluginImpl.Init;
Begin
    TfrmMain(owner).implEnvelopeReceived := onEnvelopeReceived;
    TfrmMain(owner).implEnvelopeSent := onEnvelopeSent;
    TfrmMain(owner).implEnvelopeDeleted := onEnvelopeDeleted;
End;

destructor TPluginImpl.Destroy;
Begin
    //free reserved memory
    timer.Free;
End;

function TPluginImpl.getNiceName : string;
Begin
   Result := 'Inbox: '+TfrmMain(owner).Pref.settings.getVal('eInDir')+eol+
             'Outbox: '+TfrmMain(owner).Pref.settings.getVal('eOutDir');
End;

procedure TPluginImpl.connect;
var port,iperiod : integer;
    ports,period : string;
Begin
    url := TfrmMain(owner).Pref.settings.getVal('eURL');
    period := TfrmMain(owner).Pref.settings.getVal('eCheckPeriod');
    iperiod := ozStrToInt(period,5)*1000;
    Timer.interval := iperiod;
    Timer.enabled := true;
    TfrmMain(owner).signalConnected;
end;

procedure TPluginImpl.disconnect;
Begin
    TfrmMain(owner).signalNotConnected;
    Timer.enabled := false;
End;

procedure TPluginImpl.Timer1Timer(Sender: TObject);
var period : string;
    iperiod : integer;
Begin
    Timer.enabled := false;
    onCheckDirTimer;
    period := TfrmMain(owner).Pref.settings.getVal('eCheckPeriod');
    iperiod := ozStrToInt(period,5)*1000;
    Timer.interval := iperiod;
    Timer.enabled := true;
End;

function TPluginImpl.findFile(id:string) : string;
var
    res,p1 : integer;
    FileSearchRec : TSearchRec;
    fn,tfn,sfn : string;
    tl : TStringList;
    env : TEnvelope;
    ret,sentdir : string;
    closerec : boolean;
Begin
    ret := '';
    sentDir   := TfrmMain(owner).Pref.settings.getVal('eSentDir');
    sentDir   := BackSlash(sentDir,_ADD);
    if (length(sentDir)>1) and (DirectoryExists(sentDir)) then begin
        res := FindFirst(sentdir + '*.sending', faAnyFile , FileSearchRec);
        closerec := false;
        while res = 0 do begin
           fn := FileSearchRec.Name;
           tfn := chopend(fn,8);
           p1 := backpos('.',tfn);
           sfn := leftstr(tfn,p1);
           tfn := chopfront(tfn,p1+1);
           if tfn=id then ret := sentdir+sfn;
           //find next file
           res :=  FindNext(FileSearchRec);
           closerec := true;
        end; //while
        if closeRec then FindClose(FileSearchRec);
    end;
    Result := ret;
End;

procedure TPluginImpl.onCheckDirTimer;
var
    res : integer;
    outDir,sentDir,filetype,fn,ext : string;
    tl : TStringLIst;
    env  : TEnvelope;
    gm : TGsmSMS;
    op,xml,line   : string;
    x : integer;
    closerec : boolean;
    FileSearchRec : TSearchRec;
    tfs : TFileStream;
    eext,bext : string;
Begin
    eext := '.'+TfrmMain(owner).Pref.settings.getVal('eExt');
    outDir   := TfrmMain(owner).Pref.settings.getVal('eOutDir');
    outDir   := BackSlash(outDir,_ADD);
    sentDir   := TfrmMain(owner).Pref.settings.getVal('eSentDir');
    sentDir   := BackSlash(sentDir,_ADD);
    filetype := TfrmMain(owner).Pref.settings.getVal('gbFileFormat');
    op       := TfrmMain(owner).Pref.settings.getVal('eOperator');
    if (length(outDir)>1) and (DirectoryExists(outDir)) and
       (length(sentDir)>1) and (DirectoryExists(sentDir))
        then begin
        res := FindFirst(outdir + '*.*', faAnyFile , FileSearchRec);
        closerec := false;
        while res = 0 do begin
           fn := FileSearchRec.Name;
           ext := ExtractFileExt(fn);
           bext := chopend(fn,length(ext));
           if ((ext='.sml') or (ext='.sms') or (ext=eext)) and
              (not fileexists(outdir+fn+'.lock')) and
              (not fileexists(outdir+fn+'.lck')) and
              (not fileexists(outdir+bext+'.lock')) and              
              (not fileexists(outdir+bext+'.lck')) and
              ((not (pos('.lck',fn)>0)) and (not (pos('.lock',fn)>0)))
             then begin
             TfrmMain(owner).addtolog('Found file '+outdir+fn);
             try
             tl := TStringList.Create;
             tfs := TFileStream.Create(outdir+fn,fmOpenRead or fmShareExclusive);
             tl.LoadFromStream(tfs);
             tfs.Free;
             env := TEnvelope.create;
             env.operator := op;
             env.messagetype := 'GSMSMS';
             gm := TGsmSMS.create;
             if tl.count>0 then begin
             if (ext='.sml') or ((filetype='rbList') and (ext=eext)) then begin
                for x := 1 to tl.Count do begin
                   env.makeID;
                   env.newfilename;
                   line := tl[x-1];
                   env.recepient := splithead(' ',line);
                   if toPHoneNumber(env.recepient)='' then TfrmMain(owner).addtolog('Warning: "'+env.recepient+'" does not seem like a valid phone number');
                   env.subject   := line;
                   gm.Receiver   := env.recepient;
                   gm.Msg        := env.subject;
                   env.messagedata := gm.toXML;
                   xml := env.asXml(true);
                   tl[x-1] := env.id+' '+tl[x-1];
                   TfrmMain(owner).sendEnvelope(xml);
                end;
                //move file to sent items dir
                writeToFile(sentDir+fn+'.sending',tl.Text);
                deleteFile(outdir+fn);
             end else begin
                if filetype='rbXML' then begin
                   env.fromXml(tl.text);
                   xml := env.asXml(true);
                   TfrmMain(owner).sendEnvelope(xml);
                end else if filetype='rbSimple' then begin
                   env.recepient := extractfilename(fn);
                   env.recepient := chopend(env.recepient,length(ext));
                   env.subject := tl[0];
                   gm.Receiver   := env.recepient;
                   gm.Msg        := env.subject;
                   env.messagedata := gm.toXML;
                   xml := env.asXml(true);
                   TfrmMain(owner).sendEnvelope(xml);
                end else if filetype='rbVerbouse' then begin
                   env.recepient := tl[1];
                   env.subject := tl[2];
                   gm.Receiver   := env.recepient;
                   gm.Msg        := env.subject;
                   env.messagedata := gm.toXML;
                   xml := env.asXml(true);
                   TfrmMain(owner).sendEnvelope(xml);
                end;
                //move file to sent items dir
                writeToFile(sentDir+fn+'.'+env.id+'.sending',tl.Text);
                deleteFile(outdir+fn);
             end;
             end;
             //free memory
             gm.Free;
             env.Free;
             tl.Free;
             except
               on e : exception do TfrmMain(owner).addtolog('Error processing file: '+e.message);
             end;
           end;
           //find next file
           res :=  FindNext(FileSearchRec);
           closerec := true;
        end; //while
        if closeRec then FindClose(FileSearchRec);
    end;
End;

procedure TPluginImpl.onEnvelopeSent(envelopeID : string);
var fn : string;
Begin
   try
   fn := findFile(envelopeID);
   if fn<>'' then begin
      renameFile(fn+'.'+envelopeID+'.sending',fn);
   end;
   except
      on e:exception do TfrmMain(owner).addtolog('Error moving file: '+e.message);
   end;
End;

procedure TPluginImpl.onEnvelopeDeleted(envelopeID : string);
var fn,fname,faileddir,filetype,filename,msg : string;
    env : TEnvelope;
Begin
   try           
   fn := findFile(envelopeID);
   if fn<>'' then begin
      failedDir   := TfrmMain(owner).Pref.settings.getVal('eFailedDir');
      failedDir   := BackSlash(failedDir,_ADD);
      if (length(failedDir)>1) and (DirectoryExists(failedDir)) then begin
         fname := extractfilename(fn);
         renameFile(fn+'.'+envelopeID+'.sending',failedDir+fn);
      end else begin
         renameFile(fn+'.'+envelopeID+'.sending',fn+'.failed');
      end;
   end else begin
      filetype := TfrmMain(owner).Pref.settings.getVal('gbFileFormat');
      failedDir   := TfrmMain(owner).Pref.settings.getVal('eFailedDir');
      env := TfrmMain(owner).getMessageById(envelopeID);
      if assigned(env) and (filetype='rbList') then begin
          if directoryExists(failedDir) then begin
            filename := failedDir+'\failed.sml';
            ozTouch(filename+'.lock');
            msg := env.recepient+' '+env.subject+eol;
            appendToFile(filename,msg);
            deleteFile(filename+'.lock');
            TfrmMain(owner).addtolog('Message added to file '+filename);
          end else begin
            TfrmMain(owner).addtolog('Cannot find directory '+failedDir);
          end;
      end;
      if assigned(env) then env.Free;
   end;
   except
      on e:exception do TfrmMain(owner).addtolog('Error moving file: '+e.message);
   end;

End;

procedure TPluginImpl.onEnvelopeReceived(envelopeXML : string);
var
    env : TEnvelope;
    xml,params,rec : String;
    downloaded,msg,ext : string;
    http : TozHTTPClient;
    gm : TGsmSMS;
    respond : boolean;
    filetype,filename,inDir : string;
Begin
    inDir := TfrmMain(owner).Pref.settings.getVal('eInDir');
    inDir := BackSlash(inDir,_ADD);
    filetype :=  TfrmMain(owner).Pref.settings.getVal('gbFileFormat');
    if (length(inDir)>1) and (DirectoryExists(inDir)) then begin
        try
        env := TEnvelope.Create;
        env.fromXml(envelopexml);
        if env.messagetype = 'GSMSMS' then ext := '.sms' else ext := '.msg';

        if filetype = 'rbXML' then begin
            filename := InDir+randomstringL(8)+ext;
            ozTouch(filename+'.lock');
            writeToFile(filename,envelopeXML);
            deleteFile(filename+'.lock');
            TfrmMain(owner).addtolog('XML file saved as '+indir+filename);
        end else if filetype = 'rbSimple' then begin
            filename := InDir+env.originator+ext;
            ozTouch(filename+'.lock');
            msg := env.subject+eol+nowstr;
            writeToFile(filename,msg);
            deleteFile(filename+'.lock');
            TfrmMain(owner).addtolog('Simple file saved as '+indir+filename);
        end else if filetype = 'rbVerbouse' then begin
            filename := InDir+randomstringL(8)+ext;
            ozTouch(filename+'.lock');
            msg := env.originator+eol+
                   env.recepient+eol+
                   env.subject+eol+
                   nowstr+eol;
            writeToFile(filename,msg);
            deleteFile(filename+'.lock');
            TfrmMain(owner).addtolog('Verbouse file saved as '+indir+filename);
        end else if filetype = 'rbList' then begin
            filename := InDir+'inbox.sml';
            ozTouch(filename+'.lock');
            msg := env.originator+' '+env.subject+eol;
            appendToFile(filename,msg);
            deleteFile(filename+'.lock');
            TfrmMain(owner).addtolog('Message added to file '+indir+filename);
        end;
        except
           on e:exception do TfrmMain(owner).addtolog('Error saving file: '+e.message);
        end;
        env.Free;
    end else begin
       TfrmMain(owner).addtolog('Cannot find inbox directory: '+indir);
    end;
End;

end.
